@@ -1,8 +1,8 @@
-/* $Id: minissdp.c,v 1.104 2021/05/22 21:34:10 nanard Exp $ */
+/* $Id: minissdp.c,v 1.108 2024/05/16 00:15:56 nanard Exp $ */
 /* vim: tabstop=4 shiftwidth=4 noexpandtab
  * MiniUPnP project
  * http://miniupnp.free.fr/ or https://miniupnp.tuxfamily.org/
- * (c) 2006-2021 Thomas Bernard
+ * (c) 2006-2024 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -524,11 +524,10 @@ SendSSDPResponse(int s, const struct sockaddr * addr,
 	 * uppercase is recommended.
 	 * DATE: is recommended
 	 * SERVER: OS/ver UPnP/1.0 miniupnpd/1.0
-	 * - check what to put in the 'Cache-Control' header
-	 *
-	 * have a look at the document "UPnP Device Architecture v1.1 */
+	 * CACHE-CONTROL: Should be greater than or equal to 1800 seconds
+	 */
 	l = snprintf(buf, sizeof(buf), "HTTP/1.1 200 OK\r\n"
-		"CACHE-CONTROL: max-age=120\r\n"
+		"CACHE-CONTROL: max-age=1800\r\n"
 #ifdef ENABLE_HTTP_DATE
 		"DATE: %s\r\n"
 #endif
@@ -613,7 +612,9 @@ static struct {
 	{"urn:schemas-upnp-org:device:WANConnectionDevice:", 2, uuidvalue_wcd},
 	{"urn:schemas-upnp-org:device:WANDevice:", 2, uuidvalue_wan},
 	{"urn:schemas-upnp-org:service:WANIPConnection:", 2, uuidvalue_wcd},
+#ifdef ENABLE_DP_SERVICE
 	{"urn:schemas-upnp-org:service:DeviceProtection:", 1, uuidvalue_igd},
+#endif
 #ifdef ENABLE_6FC_SERVICE
 	{"urn:schemas-upnp-org:service:WANIPv6FirewallControl:", 1, uuidvalue_wcd},
 #endif
@@ -914,7 +915,11 @@ ProcessSSDPRequest(int s, unsigned short http_port)
 		   errno != EWOULDBLOCK &&
 		   errno != EINTR)
 		{
+#if defined(IP_RECVIF) || defined(IP_PKTINFO)
+			syslog(LOG_ERR, "recvmsg(udp): %m");
+#else
 			syslog(LOG_ERR, "recvfrom(udp): %m");
+#endif
 		}
 		return;
 	}
@@ -1166,6 +1171,18 @@ ProcessSSDPData(int s, const char *bufr, int n,
 				/* retrieve the IPv6 address which
 				 * will be used locally to reach sender */
 				memset(&addr6, 0, sizeof(addr6));
+				/* UPnP Device Architecture 2.0 - Annex A - IP version 6 support
+				 * p112 A2.3 :
+				 * a) Devices and control points SHALL use only the Link-Local
+				 * unicast address as the source address and when specifying
+				 * a literal IP address in LOCATION URLs in all multicast
+				 * messages that are multicast to the Link-Local scope FF02::C
+				 * for SSDP and FF02::130 for multicast eventing.
+				 * f) Devices and control points SHALL use an acquired ULA or
+				 * GUA in all multicast messages as the source address and
+				 * when specifying a literal IP address in LOCATION URLs
+				 * that are multicast to the Site-Local scope addresses of
+				 * either FF05::C or FF05::130 */
 				if(IN6_IS_ADDR_LINKLOCAL(&(((struct sockaddr_in6 *)sender)->sin6_addr))) {
 					get_link_local_addr(((struct sockaddr_in6 *)sender)->sin6_scope_id, &addr6);
 				} else if(get_src_for_route_to (sender, &addr6, &addr6_len, &index) < 0) {
@@ -1173,6 +1190,7 @@ ProcessSSDPData(int s, const char *bufr, int n,
 					announced_host = ipv6_addr_for_http_with_brackets;
 				}
 				if(announced_host == NULL) {
+					/* convert addr6 to string with brackets */
 					if(inet_ntop(AF_INET6, &addr6,
 					             announced_host_buf+1,
 					             sizeof(announced_host_buf) - 2)) {
