@@ -1,10 +1,10 @@
-/* $Id: nftnlrdr_misc.c,v 1.18 2023/01/19 23:15:03 nanard Exp $ */
+/* $Id: nftnlrdr_misc.c,v 1.20 2024/03/19 23:35:54 nanard Exp $ */
 /* vim: tabstop=4 shiftwidth=4 noexpandtab
  * MiniUPnP project
  * http://miniupnp.free.fr/ or https://miniupnp.tuxfamily.org/
  * (c) 2015 Tomofumi Hayashi
  * (c) 2019 Paul Chambers
- * (c) 2019-2023 Thomas Bernard
+ * (c) 2019-2024 Thomas Bernard
  *
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution.
@@ -68,6 +68,9 @@ const char * nft_nat_table = "filter";
 const char * nft_prerouting_chain = "prerouting_miniupnpd";
 const char * nft_postrouting_chain = "postrouting_miniupnpd";
 const char * nft_forward_chain = "miniupnpd";
+int nft_nat_family = NFPROTO_INET;
+int nft_ipv4_family = NFPROTO_INET;
+int nft_ipv6_family = NFPROTO_INET;
 
 static struct mnl_socket *mnl_sock = NULL;
 static uint32_t mnl_portid = 0;
@@ -126,8 +129,8 @@ void
 print_rule_t(const char *func, int line, const rule_t *r)
 {
 	fprintf(stdout, "%s[%d]: ", func, line);
-	printf("%s %s %d %hu => %hu\n", r->table, r->chain, (int)r->type,
-	       r->eport, r->iport);
+	printf("%s %s %d %hu => %hu => %hu\n", r->table, r->chain, (int)r->type,
+	       r->sport, r->dport, r->nat_port);
 }
 
 /* print out the "filter" and "nat" tables */
@@ -297,14 +300,9 @@ parse_rule_nat(struct nftnl_expr *e, rule_t *r)
 	}
 	reg_val_ptr = get_reg_val_ptr(r, addr_min_reg);
 	if (reg_val_ptr != NULL) {
-		if (r->nat_type == NFT_NAT_DNAT) {
-			r->iaddr = (in_addr_t)*reg_val_ptr;
-			r->iport = proto_min_val;
-		} else if (r->nat_type == NFT_NAT_SNAT) {
-			r->eaddr = (in_addr_t)*reg_val_ptr;
-			if (proto_min_reg == NFT_REG_1) {
-				r->eport = proto_min_val;
-			}
+		r->nat_addr = (in_addr_t)*reg_val_ptr;
+		if (proto_min_reg == NFT_REG_1) {
+			r->nat_port = proto_min_val;
 		}
 	} else {
 		syslog(LOG_ERR, "%s: invalid addr_min_reg %u", "parse_rule_nat", addr_min_reg);
@@ -356,15 +354,28 @@ parse_rule_payload(struct nftnl_expr *e, rule_t *r)
 		} else if (offset == offsetof(struct ipv6hdr, saddr) &&
 			   len == sizeof(struct in6_addr) * 2) {
 			*regptr = RULE_REG_IP6_SD_ADDR;
+		} else {
+			syslog(LOG_WARNING,
+				   "%s: Unsupported payload: (dreg:%u, base:NETWORK_HEADER, offset:%u, len:%u)",
+				   "parse_rule_payload", dreg, offset, len);
 		}
 		break;
 	case NFT_PAYLOAD_TRANSPORT_HEADER:
+		/* in both UDP and TCP headers, source port is at offset 0,
+		 * destination port at offset 2 */
 		if (offset == offsetof(struct tcphdr, dest) &&
 		    len == sizeof(uint16_t)) {
 			*regptr = RULE_REG_TCP_DPORT;
+		} else if (offset == offsetof(struct tcphdr, source) &&
+			   len == sizeof(uint16_t)) {
+			*regptr = RULE_REG_TCP_SPORT;
 		} else if (offset == offsetof(struct tcphdr, source) &&
 			   len == sizeof(uint16_t) * 2) {
 			*regptr = RULE_REG_TCP_SD_PORT;
+		} else {
+			syslog(LOG_WARNING,
+				   "%s: Unsupported payload: (dreg:%u, base:TRANSPORT_HEADER, offset:%u, len:%u)",
+				   "parse_rule_payload", dreg, offset, len);
 		}
 		break;
 	default:
@@ -415,44 +426,36 @@ parse_rule_cmp(struct nftnl_expr *e, rule_t *r)
 		break;
 	case RULE_REG_IP_SRC_ADDR:
 		if (data_len == sizeof(in_addr_t)) {
-			r->rhost = *(const in_addr_t *)data_val;
+			r->saddr = *(const in_addr_t *)data_val;
 		}
 		break;
 	case RULE_REG_IP6_SRC_ADDR:
 		if (data_len == sizeof(struct in6_addr)) {
-			r->rhost6 = *(const struct in6_addr *)data_val;
+			r->saddr6 = *(const struct in6_addr *)data_val;
 		}
 		break;
 	case RULE_REG_IP_DEST_ADDR:
 		if (data_len == sizeof(in_addr_t)) {
-			if (r->type == RULE_FILTER) {
-				r->iaddr = *(const in_addr_t *)data_val;
-			} else {
-				r->rhost = *(const in_addr_t *)data_val;
-			}
+			r->daddr = *(const in_addr_t *)data_val;
 		}
 		break;
 	case RULE_REG_IP6_DEST_ADDR:
 		if (data_len == sizeof(struct in6_addr)) {
-			if (r->type == RULE_FILTER) {
-				r->iaddr6 = *(const struct in6_addr *)data_val;
-			} else {
-				r->rhost6 = *(const struct in6_addr *)data_val;
-			}
+			r->daddr6 = *(const struct in6_addr *)data_val;
 		}
 		break;
 	case RULE_REG_IP_SD_ADDR:
 		if (data_len == sizeof(in_addr_t) * 2) {
 			const in_addr_t *addrp = (const in_addr_t *)data_val;
-			r->iaddr = addrp[0];
-			r->rhost = addrp[1];
+			r->saddr = addrp[0];
+			r->daddr = addrp[1];
 		}
 		break;
 	case RULE_REG_IP6_SD_ADDR:
 		if (data_len == sizeof(struct in6_addr) * 2) {
 			const struct in6_addr *addrp6 = (const struct in6_addr *)data_val;
-			r->iaddr6 = addrp6[0];
-			r->rhost6 = addrp6[1];
+			r->saddr6 = addrp6[0];
+			r->daddr6 = addrp6[1];
 		}
 		break;
 	case RULE_REG_IP_PROTO:
@@ -461,16 +464,21 @@ parse_rule_cmp(struct nftnl_expr *e, rule_t *r)
 			r->proto = *(const uint8_t *)data_val;
 		}
 		break;
+	case RULE_REG_TCP_SPORT:
+		if (data_len == sizeof(uint16_t)) {
+			r->sport = ntohs(*(const uint16_t *)data_val);
+		}
+		break;
 	case RULE_REG_TCP_DPORT:
 		if (data_len == sizeof(uint16_t)) {
-			r->eport = ntohs(*(const uint16_t *)data_val);
+			r->dport = ntohs(*(const uint16_t *)data_val);
 		}
 		break;
 	case RULE_REG_TCP_SD_PORT:
 		if (data_len == sizeof(uint16_t) * 2) {
 			const uint16_t * ports = (const uint16_t *)data_val;
-			r->eport = ntohs(ports[0]);
-			r->rport = ntohs(ports[1]);
+			r->sport = ntohs(ports[0]);
+			r->dport = ntohs(ports[1]);
 		}
 		break;
 	default:
@@ -628,7 +636,7 @@ int
 refresh_nft_cache_filter(void)
 {
 	if (rule_list_filter_validate != RULE_CACHE_VALID) {
-		if (refresh_nft_cache(&head_filter, nft_table, nft_forward_chain, NFPROTO_INET, RULE_FILTER) < 0)
+		if (refresh_nft_cache(&head_filter, nft_table, nft_forward_chain, nft_ipv4_family, RULE_FILTER) < 0)
 			return -1;
 		rule_list_filter_validate = RULE_CACHE_VALID;
 	}
@@ -639,7 +647,7 @@ int
 refresh_nft_cache_peer(void)
 {
 	if (rule_list_peer_validate != RULE_CACHE_VALID) {
-		if (refresh_nft_cache(&head_peer, nft_nat_table, nft_postrouting_chain, NFPROTO_INET, RULE_NAT) < 0)
+		if (refresh_nft_cache(&head_peer, nft_nat_table, nft_postrouting_chain, nft_nat_family, RULE_NAT) < 0)
 			return -1;
 		rule_list_peer_validate = RULE_CACHE_VALID;
 	}
@@ -650,7 +658,7 @@ int
 refresh_nft_cache_redirect(void)
 {
 	if (rule_list_redirect_validate != RULE_CACHE_VALID) {
-		if (refresh_nft_cache(&head_redirect, nft_nat_table, nft_prerouting_chain, NFPROTO_INET, RULE_NAT) < 0)
+		if (refresh_nft_cache(&head_redirect, nft_nat_table, nft_prerouting_chain, nft_nat_family, RULE_NAT) < 0)
 			return -1;
 		rule_list_redirect_validate = RULE_CACHE_VALID;
 	}
@@ -1055,6 +1063,13 @@ rule_set_filter(uint8_t family, const char * ifname, uint8_t proto,
 	return r;
 }
 
+/*
+ * Create the IP6 filter rule
+ * called by add_pinhole() and update_pinhole()
+ * eport is always 0
+ * iport is the destination port of the filter rule
+ * rport is the (optional) source port of the rule
+ */
 struct nftnl_rule *
 rule_set_filter6(uint8_t family, const char * ifname, uint8_t proto,
 		struct in6_addr *rhost6, struct in6_addr *iaddr6,
@@ -1095,6 +1110,12 @@ rule_set_filter6(uint8_t family, const char * ifname, uint8_t proto,
 	return r;
 }
 
+/*
+ * Create common parts for the filter rules (IPv4 or IPv6)
+ * eport is ignored
+ * iport is the destination port of the filter rule
+ * rport is the (optional) source port of the rule
+ */
 struct nftnl_rule *
 rule_set_filter_common(struct nftnl_rule *r, uint8_t family, const char * ifname,
 		uint8_t proto, unsigned short eport, unsigned short iport,
@@ -1166,7 +1187,7 @@ rule_del_handle(rule_t *rule)
 
 	if (rule->type == RULE_NAT) {
 		// NAT Family is not chain/rule family
-		nftnl_rule_set_u32(r, NFTNL_RULE_FAMILY, NFPROTO_INET);
+		nftnl_rule_set_u32(r, NFTNL_RULE_FAMILY, nft_nat_family);
 	} else {
 		nftnl_rule_set_u32(r, NFTNL_RULE_FAMILY, rule->family);
 	}
