@@ -1,8 +1,8 @@
-/* $Id: upnpglobalvars.c,v 1.46 2020/09/28 21:48:48 nanard Exp $ */
+/* $Id: upnpglobalvars.c,v 1.48 2021/08/21 08:15:00 nanard Exp $ */
 /* vim: tabstop=4 shiftwidth=4 noexpandtab
  * MiniUPnP project
  * http://miniupnp.free.fr/ or https://miniupnp.tuxfamily.org/
- * (c) 2006-2020 Thomas Bernard
+ * (c) 2006-2021 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -28,6 +28,9 @@ uint16_t ext_stun_port = 0;
 /* file to store leases */
 #ifdef ENABLE_LEASEFILE
 const char* lease_file = 0;
+#ifdef ENABLE_UPNPPINHOLE
+const char* lease_file6 = 0;
+#endif
 #endif
 
 /* forced ip address to use for this interface
@@ -157,3 +160,7 @@ unsigned int upnp_configid = 1337; /* CONFIGID.UPNP.ORG */
 #ifdef RANDOMIZE_URLS
 char random_url[RANDOM_URL_MAX_LEN] = "random";
 #endif /* RANDOMIZE_URLS */
+
+#ifdef DYNAMIC_OS_VERSION
+char * os_version = NULL;
+#endif /* DYNAMIC_OS_VERSION */
